@@ -1,20 +1,35 @@
+from chalk._monitoring.Chart import Chart, Formula, Series, Trigger
+from chalk._version import __version__
 from chalk.features import Cron, Environments, Tags, description, is_primary, owner, tags
 from chalk.features.resolver import OfflineResolver, OnlineResolver, Resolver, offline, online
 from chalk.logging import chalk_logger
 from chalk.state import State
 from chalk.streams import stream
-from chalk.utils.duration import ScheduleOptions
+from chalk.utils import AnyDataclass
+from chalk.utils.duration import CronTab, Duration, ScheduleOptions
 
 batch = offline
 realtime = online
 
 __all__ = [
+    "AnyDataclass",
+    "Chart",
     "Cron",
+    "CronTab",
+    "Duration",
     "Environments",
+    "Formula",
+    "OfflineResolver",
+    "OnlineResolver",
+    "Resolver",
     "ScheduleOptions",
+    "Series",
     "State",
     "Tags",
+    "Trigger",
+    "__version__",
     "batch",
+    "chalk_logger",
     "description",
     "is_primary",
     "offline",
@@ -23,8 +38,4 @@ __all__ = [
     "realtime",
     "stream",
     "tags",
-    "Resolver",
-    "chalk_logger",
-    "OnlineResolver",
-    "OfflineResolver",
 ]
