@@ -4,17 +4,20 @@ import os
 import sys
 import traceback
 from pathlib import Path
-from typing import Callable, List
+from typing import Callable, List, Optional
 
+from chalk.features.resolver import Resolver, StreamResolver
 from chalk.gitignore.gitignore_parser import parse_gitignore
 from chalk.parsed.duplicate_input_gql import FailedImport
+from chalk.sql._internal.sql_file_resolver import ResolverResult, get_sql_file_resolvers
+from chalk.sql._internal.sql_source import BaseSQLSource
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.paths import get_directory_root
 
 _logger = get_logger(__name__)
 
 
-def py_path_to_module(path: Path, repo_root: Path) -> str:
+def _py_path_to_module(path: Path, repo_root: Path) -> str:
     try:
         p = path.relative_to(repo_root)
     except ValueError:
@@ -22,8 +25,8 @@ def py_path_to_module(path: Path, repo_root: Path) -> str:
     return str(p)[: -len(".py")].replace("./", "").replace("/", ".")
 
 
-def import_all_python_files() -> List[FailedImport]:
-    project_root = get_directory_root()
+def _import_all_files() -> List[FailedImport]:
+    project_root: Optional[Path] = get_directory_root()
     if project_root is None:
         return [
             FailedImport(
@@ -32,7 +35,30 @@ def import_all_python_files() -> List[FailedImport]:
                 traceback="Could not find chalk.yaml in this directory or any parent directory",
             )
         ]
-    return import_all_python_files_from_dir(project_root=project_root)
+    failed_imports: List[FailedImport] = import_all_python_files_from_dir(project_root=project_root)
+    failed_imports.extend(import_sql_file_resolvers(project_root))
+    return failed_imports
+
+
+def import_sql_file_resolvers(path: Path):
+    sql_resolver_results: List[ResolverResult] = list(get_sql_file_resolvers(path, BaseSQLSource.registry))
+    failed_imports: List[FailedImport] = []
+    for result in sql_resolver_results:
+        if result.resolver:
+            if isinstance(result.resolver, StreamResolver):
+                StreamResolver.registry.append(result.resolver)
+            else:
+                Resolver.registry.append(result.resolver)
+        if result.errors:
+            for error in result.errors:
+                failed_imports.append(
+                    FailedImport(
+                        traceback=error.display,
+                        filename=error.path,
+                        module=error.path,
+                    )
+                )
+    return failed_imports
 
 
 def _search_recursively_for_file(base: Path, filename: str) -> List[Path]:
@@ -76,7 +102,7 @@ def import_all_python_files_from_dir(project_root: Path) -> List[FailedImport]:
 
     venv = os.environ.get("VIRTUAL_ENV")
     module_paths = [
-        (py_path_to_module(repo_file, repo_root), repo_file)
+        (_py_path_to_module(repo_file, repo_root), repo_file)
         for repo_file in repo_files
         if venv is None or Path(venv) not in repo_file.parents
     ]
