@@ -1,62 +1,53 @@
+from __future__ import annotations
+
 import contextlib
+import csv
 import io
-from typing import Any, Dict, Mapping, Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Mapping, Optional, Union
 
 import pyarrow as pa
 import pyarrow.csv
-from sqlalchemy import (
-    BIGINT,
-    BOOLEAN,
-    CHAR,
-    DATETIME,
-    FLOAT,
-    INTEGER,
-    SMALLINT,
-    TEXT,
-    TIMESTAMP,
-    VARCHAR,
-    BigInteger,
-    Boolean,
-    Date,
-    DateTime,
-    Float,
-    Integer,
-    SmallInteger,
-    String,
-    Text,
-)
-from sqlalchemy.engine import URL, Connection
-from sqlalchemy.sql import Select
 
+from chalk.features import FeatureConverter
 from chalk.integrations.named import load_integration_variable
 from chalk.sql._internal.sql_source import BaseSQLSource, TableIngestMixIn, validate_dtypes_for_efficient_execution
 from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import SQLSourceWithTableIngestProtocol
+from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
 
-_SUPPORTED_SQLALCHEMY_TYPES_FOR_PA_CSV_QUERYING = (
-    BigInteger,
-    Boolean,
-    Float,
-    Integer,
-    String,
-    Text,
-    DateTime,
-    Date,
-    SmallInteger,
-    BIGINT,
-    BOOLEAN,
-    CHAR,
-    DATETIME,
-    FLOAT,
-    INTEGER,
-    SMALLINT,
-    TEXT,
-    TIMESTAMP,
-    VARCHAR,
-)
+if TYPE_CHECKING:
+    from sqlalchemy.engine import URL, Connection
 
-from chalk.utils.log_with_context import get_logger
+try:
+    import sqlalchemy as sa
+except ImportError:
+    sa = None
+
+if sa is None:
+    _supported_sqlalchemy_types_for_pa_csv_querying = ()
+else:
+    _supported_sqlalchemy_types_for_pa_csv_querying = (
+        sa.BigInteger,
+        sa.Boolean,
+        sa.Float,
+        sa.Integer,
+        sa.String,
+        sa.Text,
+        sa.DateTime,
+        sa.Date,
+        sa.SmallInteger,
+        sa.BIGINT,
+        sa.BOOLEAN,
+        sa.CHAR,
+        sa.DATETIME,
+        sa.FLOAT,
+        sa.INTEGER,
+        sa.SMALLINT,
+        sa.TEXT,
+        sa.TIMESTAMP,
+        sa.VARCHAR,
+    )
 
 _logger = get_logger(__name__)
 
@@ -103,6 +94,8 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         )
 
     def local_engine_url(self) -> URL:
+        from sqlalchemy.engine.url import URL
+
         return URL.create(
             drivername="postgresql",
             username=self.user,
@@ -113,13 +106,17 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         )
 
     def execute_query_efficient(
-        self, finalized_query: FinalizedChalkQuery, connection: Optional[Connection] = None
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Connection] = None,
     ) -> pa.Table:
         import psycopg2.sql
+        from sqlalchemy.sql import Select
 
         if isinstance(finalized_query.query, Select):
             validate_dtypes_for_efficient_execution(
-                finalized_query.query, _SUPPORTED_SQLALCHEMY_TYPES_FOR_PA_CSV_QUERYING
+                finalized_query.query, _supported_sqlalchemy_types_for_pa_csv_querying
             )
         stmt, positional_params, named_params = self.compile_query(finalized_query, paramstyle="named")
         assert len(positional_params) == 0, "should not have any positional params"
@@ -137,6 +134,16 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
                 buffer = io.BytesIO()
                 cursor.copy_expert(formatted_stmt, buffer)
         buffer.seek(0)
+        # Peek the column names
+        first_line = buffer.readline()
+        column_names = list(
+            csv.reader([first_line.decode("utf8")], escapechar="\\", doublequote=False, quoting=csv.QUOTE_ALL)
+        )[0]
+        buffer.seek(0)
+        converters = columns_to_converters(column_names)
+
+        schema = pa.schema([pa.field(k, v.pyarrow_dtype) for (k, v) in converters.items()])
+
         return pyarrow.csv.read_csv(
             buffer,
             parse_options=pyarrow.csv.ParseOptions(
@@ -149,5 +156,6 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
                 false_values=["f"],
                 strings_can_be_null=True,
                 quoted_strings_can_be_null=False,
+                column_types=schema,
             ),
         )
