@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import contextlib
 import functools
 import inspect
@@ -5,20 +7,27 @@ import os
 import os.path
 import warnings
 from dataclasses import dataclass
-from enum import Enum
 from os import PathLike
-from typing import Any, ClassVar, Dict, List, Mapping, Optional, Sequence, Set, Tuple, Type, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import pyarrow as pa
-import sqlalchemy.sql.functions
-from sqlalchemy import column, text
-from sqlalchemy.engine import URL, Connection, Engine, create_engine
-from sqlalchemy.orm import InstrumentedAttribute, Session
-from sqlalchemy.sql import Select
-from sqlalchemy.sql.elements import Label
-from sqlalchemy.types import TypeEngine
 
-from chalk.features import Feature, Features, FeatureWrapper, unwrap_feature
+from chalk.features import Feature, FeatureConverter, Features, FeatureWrapper, unwrap_feature
 from chalk.sql._internal.chalk_query import ChalkQuery
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.sql._internal.string_chalk_query import StringChalkQuery
@@ -26,6 +35,14 @@ from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import BaseSQLSourceProtocol, ChalkQueryProtocol, StringChalkQueryProtocol, TableIngestProtocol
 from chalk.utils.environment_parsing import env_var_bool
 from chalk.utils.log_with_context import get_logger, get_logging_context
+from chalk.utils.missing_dependency import missing_dependency_exception
+
+if TYPE_CHECKING:
+    from sqlalchemy.engine import URL, Engine
+    from sqlalchemy.orm import Session
+    from sqlalchemy.sql import Select
+    from sqlalchemy.sql.elements import Label
+    from sqlalchemy.types import TypeEngine
 
 TTableIngestMixIn = TypeVar("TTableIngestMixIn", bound="TableIngestMixIn")
 CHALK_QUERY_LOGGING = env_var_bool("CHALK_QUERY_LOGGING")
@@ -105,6 +122,12 @@ class BaseSQLSource(BaseSQLSourceProtocol):
     registry: ClassVar[List["BaseSQLSource"]] = []
 
     def __init__(self, name: Optional[str]):
+        try:
+            import sqlalchemy
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[sql]")
+        del sqlalchemy  # unused
+
         self._incremental_settings = None
         self._resolver_and_sqlfile_to_sqlstring: Dict[Tuple[str, str], str] = {}
         self.registry.append(self)
@@ -113,7 +136,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
     def query_sql_file(
         self,
         path: Union[str, bytes, PathLike],
-        fields: Optional[Mapping[str, Union[Feature, Any]]] = None,
+        fields: Optional[Mapping[str, Union[Feature, str, Any]]] = None,
         args: Optional[Mapping[str, str]] = None,
     ) -> StringChalkQueryProtocol:
         sql_string = None
@@ -164,6 +187,8 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         return StringChalkQuery(source=self, query=query, fields=fields, params=args or {})
 
     def warm_up(self) -> None:
+        from sqlalchemy import text
+
         with self.get_engine().connect() as cnx:
             cnx.execute(text("select 1"))
 
@@ -186,19 +211,26 @@ class BaseSQLSource(BaseSQLSourceProtocol):
     def local_engine_url(self) -> URL:
         raise NotImplementedError
 
-    def execute_query(self, finalized_query: FinalizedChalkQuery, connection: Optional[Any] = None) -> pa.Table:
+    def execute_query(
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Any] = None,
+    ) -> pa.Table:
+        from sqlalchemy.engine import Connection
+
         if env_var_bool("CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION"):
-            _logger.info(
-                (
-                    f"The SQL statement will be executed into SQLAlchemy objects, as the environment "
-                    "variable 'CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION' is set. Unset this variable to execute "
-                    "the query directly into a PyArrow table."
-                )
-            )
+            pass
+            # _logger.debug(
+            #     (
+            #         f"The SQL statement will be executed into SQLAlchemy objects, as the environment "
+            #         "variable 'CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION' is set. Unset this variable to execute "
+            #         "the query directly into a PyArrow table."
+            #     )
+            # )
         else:
             try:
-
-                return self.execute_query_efficient(finalized_query, connection)
+                return self.execute_query_efficient(finalized_query, columns_to_converters, connection)
             except NotImplementedError:
                 _logger.info(
                     (
@@ -221,28 +253,32 @@ class BaseSQLSource(BaseSQLSourceProtocol):
                     )
                 )
 
-        with (self.get_engine().connect() if connection is None else contextlib.nullcontext(connection)) as cnx:
+        with self.get_engine().connect() if connection is None else contextlib.nullcontext(connection) as cnx:
             assert isinstance(cnx, Connection)
             res = cnx.execute(finalized_query.query, finalized_query.params)
-            data: Dict[str, List[Any]] = {}
             desc = res.cursor.description
             result_columns: list[str] = [col[0] for col in desc]
-            for v in result_columns:
+            converters = columns_to_converters(result_columns)
+            data: Dict[str, List[Any]] = {}
+            for v in converters.keys():
                 # Create an entry for the columns, so the pyarrow table will have the correct columns even
                 # if there is no data
                 data[v] = []
             for row in res.all():
                 for k, v in zip(result_columns, row):
-                    if isinstance(v, Enum):
-                        v = v.value
                     if k not in data:
-                        # Handle any extra columns that were somehow not in the result description
-                        data[k] = []
-                    data[k].append(v)
-            return pa.Table.from_pydict(data)
+                        # We are not interested in this column
+                        continue
+                    data[k].append(converters[k].from_rich_to_primitive(v, missing_value_strategy="default_or_allow"))
+            # Only keep the columns provided by the schema
+            schema = pa.schema([pa.field(k, v.pyarrow_dtype) for (k, v) in converters.items()])
+            return pa.Table.from_pydict(data, schema=schema)
 
     def execute_query_efficient(
-        self, finalized_query: FinalizedChalkQuery, connection: Optional[Any] = None
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Any] = None,
     ) -> pa.Table:
         raise NotImplementedError()
 
@@ -260,6 +296,8 @@ class BaseSQLSource(BaseSQLSourceProtocol):
 
     @functools.lru_cache(None)  # pyright: ignore[reportIncompatibleMethodOverride]
     def get_engine(self) -> Engine:
+        from sqlalchemy.engine import create_engine
+
         return create_engine(
             url=self.local_engine_url(),
             echo=CHALK_QUERY_LOGGING,
@@ -267,6 +305,8 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         )
 
     def raw_session(self) -> Session:
+        from sqlalchemy.orm import Session
+
         warnings.warn(
             DeprecationWarning(
                 (
@@ -280,6 +320,10 @@ class BaseSQLSource(BaseSQLSourceProtocol):
 
 
 def _extract_features(feature_set: Features, prefix: str, targets: List[Label], features: Dict[str, Feature]):
+    import sqlalchemy.sql.functions
+    from sqlalchemy import column
+    from sqlalchemy.orm import InstrumentedAttribute
+
     for f in feature_set.features:
         assert f.attribute_name is not None
         try:
