@@ -3,20 +3,25 @@ from typing import Any, List, Literal, Optional, Union
 from pydantic import BaseModel
 
 from chalk.integrations.named import load_integration_variable
+from chalk.sink._models import SinkIntegrationProtocol
 from chalk.streams.base import StreamSource
 from chalk.utils.duration import Duration
 from chalk.utils.string import comma_whitespace_split
 
 
-class KafkaSource(StreamSource, BaseModel, frozen=True):
+class KafkaSource(StreamSource, SinkIntegrationProtocol, BaseModel, frozen=True):
     bootstrap_server: Optional[Union[str, List[str]]] = None
+    """The URL of one of your Kafka brokers from which to fetch initial metadata about your Kafka cluster"""
+
     topic: Optional[Union[str, List[str]]] = None
+    """The name of the topic to subscribe to."""
+
     ssl_keystore_location: Optional[str] = None
     """
     An S3 or GCS URI that points to the keystore file that should be
-        used for brokers. You must configure the appropriate AWS or
-        GCP integration in order for Chalk to be able to access these
-        files.
+    used for brokers. You must configure the appropriate AWS or
+    GCP integration in order for Chalk to be able to access these
+    files.
     """
 
     client_id_prefix: str = "chalk/"
@@ -25,33 +30,31 @@ class KafkaSource(StreamSource, BaseModel, frozen=True):
     security_protocol: Literal["PLAINTEXT", "SSL", "SASL_PLAINTEXT", "SASL_SSL"] = "PLAINTEXT"
     """
     Protocol used to communicate with brokers.
-            Valid values are: PLAINTEXT, SSL, SASL_PLAINTEXT, SASL_SSL.
-            Default: PLAINTEXT.
+    Valid values are `"PLAINTEXT", `"SSL"`, `"SASL_PLAINTEXT"`, and `"SASL_SSL"`.
+    Defaults to `"PLAINTEXT"`.
     """
 
     sasl_mechanism: Literal["PLAIN", "GSSAPI", "SCRAM-SHA-256", "SCRAM-SHA-512", "OAUTHBEARER"] = "PLAIN"
     """
-    Authentication mechanism when security_protocol
-            is configured for SASL_PLAINTEXT or SASL_SSL. Valid values are:
-            PLAIN, GSSAPI, SCRAM-SHA-256, SCRAM-SHA-512, OAUTHBEARER.
-            Default: PLAIN
+    Authentication mechanism when `security_protocol`
+    is configured for SASL_PLAINTEXT or SASL_SSL.
+    Valid values are `"PLAIN`", `"GSSAPI"`, `"SCRAM-SHA-256"`, `"SCRAM-SHA-512"`, `"OAUTHBEARER"`.
+    Defaults to `"PLAIN"`.
     """
 
     sasl_username: Optional[str] = None
     """
     Username for SASL PLAIN, SCRAM-SHA-256, or SCRAM-SHA-512 authentication.
-            Default: None
     """
 
     sasl_password: Optional[str] = None
     """
     Password for SASL PLAIN, SCRAM-SHA-256, or SCRAM-SHA-512 authentication.
-            Default: None
     """
 
     name: Optional[str] = None
     """
-    The name of the integration, as configured in your web dashboard portal.
+    The name of the integration, as configured in your Chalk Dashboard.
     """
 
     late_arrival_deadline: Duration = "infinity"
@@ -101,5 +104,5 @@ class KafkaSource(StreamSource, BaseModel, frozen=True):
             late_arrival_deadline=late_arrival_deadline,
         )
 
-    def config_to_json(self) -> Any:
+    def _config_to_json(self) -> Any:
         return self.json()
